module Substitutions where

import Types
import Names
import Expressions
import Constants
import List
import Utils
import PPrint
import Html
-- import Trace

type Subst = (FMap VarId Exp,Sub)

idSubst :: Subst
idSubst = (idf,idsub)

appSubst :: Subst -> Exp -> Exp
appSubst (se,st) (Var x t) = appf' se (Var x (apply st t)) x
appSubst (se,st) (Con c t) = Con c (apply st t)
appSubst s@(se,st) (Ap e1 e2 t) = Ap (appSubst s e1) (appSubst s e2) 
                                                 (apply st t)
appSubst s@(se,st) (Lam p e t) = Lam (appSubst s p) (appSubst s e) (apply st t)

compose :: Subst -> Subst -> Subst
compose s1@(se1,st1) (se2,st2) = ([ (x, appSubst s1 e) | (x,e) <- se2 ] ++
                                   [ (x, e) | (x,e) <- se1, not 
                                                 (x `elem` domf se2)],
                                  comp st1 st2 ) 
                                  where domf = map fst

singleSubst :: VarId -> Type -> Exp -> [Subst]
singleSubst x t e = [ ([(x,e)],st) | st <- tmatch t (exptype e) ]

singletype :: Type -> Type -> [Subst]
singletype s t = [(idf, st) | st <- tmatch s t]

papply s (e1,xs,e2,ys) = (appSubst s e1, xs,e2,ys)

expSubst s = etaExp . appSubst  s
exppapply s (e1,xs,e2,ys) = (expSubst s e1, xs,e2,ys) 

vardom :: Subst -> [VarId]
vardom (x,s) = map fst x

funcom :: (Exp -> Exp) -> Subst -> Subst
funcom f (se,st) = ([(x,f e) | (x,e)<- se],st)

tcompatible p e = not (null (tmatch (exptype p) (exptype e)))

prettySubst (s,f) = concat (map showAssign s)
                    where showAssign (x,t) = x ++ " := " ++ prettyExp t ++ "\n"
prettySs = concat . map ((++"\n---\n").prettySubst)

htmlSubst (s,f) = foldr (+++) noHtml (map htmlAssign s)
      where htmlAssign (x,t) = thediv ! [theclass "expression"] 
               << p << (x +++ " := " 
                          +++ (linesToHtml $ lines $ display $ ppExp n $ t))
              where n = toInteger (length x + 4)