// Effect.cpp: implementation of the CEffect class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Effect.h"
#include "FontInfo.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEffect::CEffect() : m_fName(NULL), m_win(NULL), m_progID(NULL)
{

}

CEffect::CEffect(CWindowVirt* winUI,
                 CComQIPtr<IPanImage> img,
                 std::vector<CStringProp*>* propVec,
                 std::vector<CFontProp*>*   propFontVec,
                 LPTSTR progID,
                 CViewWindow* pView,
                 HWND hWndMain) 
  : m_fName(NULL), m_image(img), m_win(winUI),
    m_propVec(propVec), m_propFontVec(propFontVec),
    m_view(pView), m_hWndMain(hWndMain)
{
    m_progID = new TCHAR[sizeof(TCHAR)*(_tcslen(progID) + 1)];
    _tcscpy(m_progID,progID);
}

CEffect::~CEffect()
{
//    ::DestroyWindow(*m_win);
    delete [] m_progID;
    if (m_win) delete m_win;
    delete m_propVec;
    delete m_propFontVec;

    m_win = NULL;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

void 
CEffect::Close()
{
    CloseWindow(*m_win);
}


void
CEffect::Save()
{
    USES_CONVERSION;
    HRESULT hr;
    IStorage* pStg;
    IStream* pStream;
    IPersistStream* pPStream;
    PWSTR fNameW = T2OLE(m_fName);
    ULONG cbWritten = 0;

    hr = 
#if 0
        // We still have to use StgCreateDocfile, since
        // we will be running the viewer under NT and Win 9x.
        StgCreateStorageEx(fNameW,
                           STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                           STGFMT_DOCFILE,
                           0,
                           NULL,
                           0,
                           IID_IStorage,
                           reinterpret_cast<void**>(&pStg));
#endif
        StgCreateDocfile(fNameW,
                         STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                         0,
                         &pStg);

    m_image->QueryInterface(&pPStream);

    if ((hr = pStg->CreateStream(L"PanImage", 
                                 STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                                 0,0, &pStream)) == S_OK) {
        ULONG  ulLength;
        LPWSTR progID = T2W(m_progID);
        unsigned char cbType = 0;
        HRESULT hr;

        // Prefixing a '1' means that a ProgID will follow, this
        // could be made a whole deal safer.
        pStream->Write(&cbType,1,&cbWritten);
        ulLength = wcslen(progID);
        pStream->Write(&ulLength,sizeof(ULONG),&cbWritten);
        pStream->Write(progID,sizeof(WCHAR)*ulLength,&cbWritten);
        hr = pPStream->Save(pStream,TRUE);

        pStream->Release();
    }

    int strLen;
    CStringProp* prop;

    if (m_propVec && (hr = pStg->CreateStream(L"UISettings", 
                                 STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                                 0,0,&pStream)) == S_OK) {

       int vecSize = m_propVec->size();
       for (int i=0; i < vecSize; i++) {
            prop = (*m_propVec)[i];
            strLen = wcslen(T2CW(prop->GetString()));
            pStream->Write(&strLen,4,&cbWritten);
            pStream->Write(T2CW(prop->GetString()),sizeof(WCHAR)*(strLen+1),&cbWritten);
        }
        pStream->Release();
    }

    if (m_propFontVec && (hr = pStg->CreateStream(L"FontSettings", 
                                 STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                                 0,0,&pStream)) == S_OK) {

       FontInfo fi;
       int vecSize = m_propFontVec->size();
       for (int i=0; i < vecSize; i++) {
            (*m_propFontVec)[i]->GetFontInfo(&fi);
            pStream->Write(&fi,sizeof(FontInfo),&cbWritten);
        }
        pStream->Release();
    }

    if (m_propFontVec && (hr = pStg->CreateStream(L"MouseInput", 
                                 STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                                 0,0,&pStream)) == S_OK) {

        struct {float x,y;} pos;
        m_view->GetMouseInput(pos.x,pos.y);
        pStream->Write(&pos,sizeof(pos),&cbWritten);
        pStream->Release();
    }

    if ((hr = pStg->CreateStream(L"ViewSize", 
                                 STGM_CREATE | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
                                 0,0,&pStream)) == S_OK) {
        RECT rc;
        SIZE sz;
        GetWindowRect(m_hWndMain,&rc);

        sz.cx = rc.right - rc.left;
        sz.cy = rc.bottom - rc.top;
        pStream->Write(&sz,sizeof(SIZE),&cbWritten);

        pStream->Release();
    }
    pStg->Release();
}


LPCTSTR CEffect::GetFileName() { return m_fName; }
void CEffect::SetFileName(LPTSTR f)
{ 
    if (m_fName) {
        delete m_fName;   
    }
    m_fName = new TCHAR[lstrlen(f)+1];
    lstrcpy(m_fName,f);
}
