module CseUtils where

import List(partition,nub,sort,(\\))
import IOExts
import DExp ( Exp )

{- BEGIN_GHC_ONLY
import PrelGHC
import Stable

type Node = StableName Exp

ptrToInt :: Exp -> Node
ptrToInt e = e `seq` unsafePerformIO $ do
    x <- makeStableName e
    return x

{-
 According to the paper which presents StableNames, an Ord
 instance is supposed to be supplied, but it doesn't appear
 to be defined in Stable...so,
-}
instance Ord (StableName a) where
   -- can go wrong should a GC strike between the eval of
   -- arg1 and arg2.
  compare x y = compare (hashStableName x) (hashStableName y)

instance Show (StableName a) where
  show x = show (hashStableName x)

   END_GHC_ONLY -}

{- BEGIN_HUGS_ONLY -}

primitive ptrToInt' "unsafePtrToInt" :: a -> Int

ptrToInt e = seq e (ptrToInt' e)

type Node = Int
{- END_HUGS_ONLY -}


-- (conal) same as fmap on Maybe
perhaps :: (a -> b) -> Maybe a -> Maybe b
perhaps f (Just a) = Just (f a)
perhaps f Nothing = Nothing

maybeFilter :: (a -> Bool) -> a -> Maybe a
maybeFilter p a = if p a then Just a else Nothing

collect :: Eq a => [(a,b)] -> [(a,[b])]
collect [] = []
collect ((a,l):x) = (a,l:ls):collect z
                    where (y,z) = partition ((==a).fst) x
                          ls = map snd y

mergeWith :: Ord b => (a->b) -> [a] -> [a] -> [a]
mergeWith f [] ys = ys
mergeWith f xs [] = xs
mergeWith f (x:xs) (y:ys) = if fx <= fy
                            then x:mergeWith f xs (y:ys)
                            else y:mergeWith f (x:xs) ys
                            where fx = f x
                                  fy = f y

mergeDown :: [Int] -> [Int] -> [Int]
mergeDown x y = reverse (nub (sort (x++y)))

sortWith :: Ord b => (a->b) -> [a] -> [a]
sortWith f [] = []
sortWith f [x] = [x]
sortWith f xs = mergeWith f (sortWith f ys) (sortWith f zs)
                where (ys,zs) = split xs

-- split "abcdefg" -> ("bdf","aceg")
split = foldr twiddle ([],[])
        where twiddle x (ys,zs) = (zs,x:ys)

-- naive topological sort. It does not seem to be worthwhile
-- to use a more sophisticated algorithm, because groups of binders
-- are fairly small

topsort :: Eq a => [(a,b,[a])] -> [(a,b)]
topsort [] = []
topsort xs = (a,b):topsort [ (c,d,cs\\[a]) | (c,d,cs) <- xs, c /= a ]
             where (a,b,_) = head [ (c,d,cs) | (c,d,cs) <- xs, not (elem c (concat (map thd3 xs))) ]
                   thd3 (a,b,c) = c

-- generate fresh identifiers
gensym :: () -> String
gensym = unsafePerformIO
           (do counter <- newIORef 0
               return (\ () -> unsafePerformIO
                               (do c <- readIORef counter
                                   writeIORef counter (c+1)
                                   return ('S':show c))))

intToNode :: Int -> Node
intToNode i = 
{- BEGIN_GHC_ONLY
 i `seq` unsafePerformIO $ do
   c <- makeStableName i
   return (unsafeCoerce# c)
   END_GHC_ONLY -}
{- BEGIN_HUGS_ONLY -}
   i
{- END_HUGS_ONLY -}

newNode :: () -> Node 
newNode = unsafePerformIO
           (do counter <- newIORef 0
               return (\ () -> unsafePerformIO
                               (do c <- readIORef counter
                                   writeIORef counter (c+1)
{- BEGIN_GHC_ONLY
				   c' <- c `seq` makeStableName c
				   return (unsafeCoerce# c')
   END_GHC_ONLY -}
{- BEGIN_HUGS_ONLY -}
                                   return c
{- END_HUGS_ONLY -}
	                                    )))

twicein :: Ord a => [[a]] -> [a]
twicein = map fst . filter ((>1).snd) . 
          foldr mergecount [] . 
          map (map (\n -> (n,1)))

mergecount :: Ord a => [(a,Int)] -> [(a,Int)] -> [(a,Int)]
mergecount xs [] = xs
mergecount [] ys = ys
mergecount xns@((x,n):xs) yns@((y,m):ys) | x==y = (x,n+m):mergecount xs ys
                                         | x< y = (x,n):mergecount xs yns
                                         | otherwise = (y,m):mergecount xns ys

-- (conal) doesn't seem correct.  shouldn't the test be sum ys > 1?
twice :: Eq a => a -> [(a,Int)] -> Bool
twice a xs = not (null ys) && head ys > 0
             where ys = [ n | (x,n) <- xs, x==a]

oin :: Ord a => [a] -> [a] -> [a]
oin [] xs = []
oin xs [] = []
oin xxs@(x:xs) yys@(y:ys) | x < y  = oin xs yys
                          | x == y = x: oin xs ys
                          | x > y  = oin xxs ys

notin :: Ord a => [a] -> [a] -> [a]
notin [] xs = xs
notin xxs@(x:xs) yys@(y:ys) | x < y = notin xs yys
                            | x== y = notin xs ys
                            | x > y = y: notin xxs ys

merge :: Ord a => [a] -> [a] -> [a]
merge [] xs = xs
merge xs [] = xs
merge xxs@(x:xs) yys@(y:ys) | x == y = x : merge xs ys
                            | x < y  = x : merge xs yys
                            | x > y  = y : merge xxs ys

myintersect :: Ord a => [a] -> [a] -> [a]
myintersect [] xs = []
myintersect xs [] =[]
myintersect xxs@(x:xs) yys@(y:ys) | x==y = x: myintersect xs ys
                                | x< y = myintersect xs yys
                                | x > y = myintersect xxs ys

insert :: Ord a => a -> [a] -> [a]
insert x [] = [x]
insert x (y:ys) | x == y = y:ys
                | x < y  = x:y:ys
                | x > y  = y:insert x ys


